#跑道长度
setting_track_length = 20
#随机位置事件，最小能到的跑道距离
setting_random_min_length = 0
#随机位置事件，最大能到的跑道距离
setting_random_max_length = 15
#每回合基础移动力最小值
base_move_min = 1
#每回合基础移动力最大值
base_move_max = 3
#最大支持玩家数
max_player = 8
#最少玩家数
min_player = 2
#超时允许重置最少时间，秒
setting_over_time = 120

#事件概率 = event_rate / 1000
event_rate = 450



import random
from .setting import  *

class horse:
    def __init__(self, horsename = "the_horse", uid = 114514, id = "the_player", location = 0, round = 0 ):
        self.horse = horsename
        self.playeruid = uid
        self.player =  id
        self.buff = []
        self.delay_events = []
        self.horse_fullname = horsename
        self.round = round
        self.location = location
        self.location_add = 0
        self.location_add_move = 0
# =====替换为其他马,指定马（没用上，暂留）
    def replace_horse(self, horse_to):
        self.horse = horse_to.horse
        self.playeruid = horse_to.playeruid
        self.player = horse_to.player
        self.buff = horse_to.buff
        self.delay_events = horse_to.delay_events
        self.horse_fullname = horse_to.horse_fullname
        self.round = horse_to.round
        self.location = horse_to.location
        self.location_add = horse_to.location_add
        self.location_add_move = horse_to.location_add_move
#=====替换为其他马,指定数据（用于天灾马系列事件）
    def replace_horse_ex(self, horsename = "the_horse", uid = 114514, id= "the_player"):
        self.horse = horsename
        self.playeruid = uid
        self.player =  id
        self.buff = []
        self.delay_events = []
        self.horse_fullname = horsename
        self.round = round
        self.location = location
        self.location_add = 0
        self.location_add_move = 0
#=====马儿buff增加：buff为list格式
    def add_buff(self, buff_name, round_start, round_end, buffs, move_min = 0, move_max = 0, event_in_buff = []):
        if  move_min > move_max:
            move_max = move_min
        buff = [buff_name, round_start, round_end, move_min, move_max, event_in_buff]
        buff.extend(buffs)
        self.buff.append(buff)
#=====马儿指定buff移除：
    def del_buff(self, del_buff_key):
        for i in range(0, len(self.buff)):
            try:
                self.buff[i].index(del_buff_key, 6)
                self.buff[i][2] = self.round
            except ValueError:
                pass
#=====马儿查找有无buff（查参数非名称）：(跳过计算回合数，只查有没有）
    def find_buff(self, find_buff_key):
        for i in range(0, len(self.buff)):
            try:
                self.buff[i].index(find_buff_key, 6)
                return True
            except ValueError:
                pass
        return False
#=====马儿超时buff移除：
    def del_buff_overtime(self, round):
        for i in range(len(self.buff) - 1, -1, -1):
            if self.buff[i][2] < round:
                del self.buff[i]
#=====马儿buff时间延长/减少：
    def buff_addtime(self, round_add):
        for i in range(0, len(self.buff)):
            self.buff[i][2] += round_add
#=====马儿是否止步：
    def is_stop(self) -> bool:
        for i in range(0, len(self.buff)):
            try:
                self.buff[i].index("locate_lock", 6)
                if self.buff[i][1] <= self.round:
                    return True
            except ValueError:
                pass
        return False
#=====马儿是否已经离开：
    def is_away(self) -> bool:
        for i in range(0, len(self.buff)):
            try:
                self.buff[i].index("away", 5)
                if self.buff[i][1] <= self.round:
                    return True 
            except ValueError:
                pass
        return False
#=====马儿是否已经死亡：
    def is_die(self) -> bool:
        for i in range(0, len(self.buff)):
            try:
                self.buff[i].index("die", 5)
                if self.buff[i][1] <= self.round:
                    return True
            except ValueError:
                pass
        return False
#=====马儿全名带buff显示：
    def fullname(self):
        fullname = f""
        for i in range(0, len(self.buff)):
            if self.buff[i][1] <= self.round:
                fullname += ( "<" + self.buff[i][0] + ">" )
        self.horse_fullname = fullname + self.horse
#=====马儿移动计算（事件提供的本回合移动）：
    def location_move_event(self, move):
        self.location_add_move += move
#=====马儿移动至特定位置计算（事件提供移动）：
    def location_move_to_event(self, move_to):
        self.location_add_move += move_to - self.location
#=====马儿移动计算：
    def location_move(self):
        if self.location != setting_track_length:
            self.location_add = self.move() + self.location_add_move
            self.location += self.location_add
            if self.location > setting_track_length:
                self.location_add -= self.location - setting_track_length
                self.location = setting_track_length
            if self.location < 0:
                self.location_add -= self.location
                self.location = 0
#=====马儿移动量计算：
    def move(self):
        if self.is_stop() == True:
            return 0
        if self.is_die() == True:
            return 0
        if self.is_away() == True:
            return 0
        move_min = 0
        move_max = 0
        for i in range(0, len(self.buff)):
            if self.buff[i][1] <= self.round:
                move_min += self.buff[i][3]
                move_max += self.buff[i][4]
        return random.randint(move_min + base_move_min, move_max + base_move_max)
#=====赛马玩家战况显示： 
    def display(self):
        display = f""
        if self.find_buff("hiding") == False:
            if self.location_add < 0:
                display += "[" + str(self.location_add) + "]"
            else:
                display += "[+" + str(self.location_add) + "]"
            for i in range(0, setting_track_length - self.location):
                display += "."
            display += self.horse_fullname
            for i in range(setting_track_length - self.location, setting_track_length):
                display += "."
        else:
            display += "[+？]"
            for i in range(0, setting_track_length):
                display += "."
        display += "\n"
        return display




import random
from .horse import horse
from .setting import  *
from nonebot.log import logger

def event_main(race, horse_i, event, event_delay_key = 0):
    try:
    #读取 event_name
        event_name = event["event_name"]
    #该马儿是否死亡/离开/眩晕，死亡则结束事件链
        if race.player[horse_i].is_die() == True and event_delay_key == 0:
            return f''
        elif race.player[horse_i].is_away() == True and event_delay_key == 0:
            return f''
        elif race.player[horse_i].find_buff("vertigo") == True and event_delay_key == 0:
            return f''
    #读取事件限定值
        if event["race_only_exist"] == 1:
            race_only_key = event["race_only"]
            if race.is_race_only_key_in(race_only_key)== True:
                return f''
            else:
                race.add_race_only_key(race_only_key)
    #读取 describe 事件描述
        describe = event["describe"]
    #读取 target 目标，计算<0><1>， target_name_0 ， target_name_1
        target = event["target"]
        targets = []
        target_name_0 = race.player[horse_i].horse_fullname
        for i in range(0, len(race.player)):
            targets.append(i)
        if target == 0:
            # 0目标为自己
            targets = [horse_i]
            target_name_1 = race.player[horse_i].horse_fullname
        elif target == 1:
            # 1为随机选择一个非自己的目标（即<1>）
    #        targets.pop(horse_i)
            del targets[horse_i]
            targets = random.sample(targets, 1)
            target_name_1 = race.player[targets[0]].horse_fullname
        elif target == 2:
            # 2为全部
            target_name_1 = "所有马儿"
        elif target == 3:
            # 3为除自身外全部
    #        targets.pop(horse_i)
            del targets[horse_i]
            target_name_1 = "其他所有马儿"
        elif target == 4:
            # 4为全场随机一个目标
            targets = random.sample(targets, 1)
            target_name_1 = race.player[targets[0]].horse_fullname
        elif target == 5:
            # 5自己和一位其他目标（该目标为<1>）
    #        targets.pop(horse_i)
            del targets[horse_i]
            a = random.sample(targets, 1)
            targets = [horse_i]
            targets.extend(a)
            target_name_1 = race.player[a[0]].horse_fullname
        elif target == 6:
            # 6为随机一侧赛道的马儿（即<1>）
            a = []
            try:
                targets.index(horse_i - 1)
                a.append(horse_i - 1)
            except:
                pass
            try:
                targets.index(horse_i + 1)
                a.append(horse_i + 1)
            except:
                pass
            targets = random.sample(a, 1)
            target_name_1 = race.player[targets[0]].horse_fullname
        elif target == 7:
            # 7为两侧赛道的马儿（即<1>，此<1>为马1和马2）,赛道最边上则结束事件链
            a = []
            try:
                targets.index(horse_i - 1)
                a.append(horse_i - 1)
            except:
                return f''
            try:
                targets.index(horse_i + 1)
                a.append(horse_i + 1)
            except:
                return f''
            targets = a
            target_name_1 = f'在{race.player[horse_i].horse_fullname}两侧的马儿'
            # 其他则结束事件链
        else:
            return f''
    #判定target_is_buff
        target_is_buff = event["target_is_buff"]
        if target_is_buff != f"":
            a = []
            for i in targets:
                if race.player[i].find_buff(target_is_buff) == True:
                    a.append(i)
            targets = a
    #判定target_no_buff
        target_no_buff = event["target_no_buff"]
        if target_no_buff != f"":
            a = []
            for i in targets:
                if race.player[i].find_buff(target_no_buff) == False:
                    a.append(i)
            targets = a
    #判定结束，无目标返回空值，结束事件
        if targets == []:
            return f''
    #目标判定完毕，目标为targets，list值
        logger.info(f'执行事件: {event_name}')
        logger.info(f'target：{str(targets)}，<0>为：{target_name_0}，<1>为：{target_name_1}')
    #describe值<0><1>解读
        describe = describe.replace('<0>', target_name_0)
        describe = describe.replace('<1>', target_name_1)

    #===============以下为一次性事件===============
    #复活事件：为1则目标复活
        if event["live"] == 1:
            event_live(race, targets)
    #位移事件：目标立即进行相当于参数值的位移
        if event["move"] != 0:
            event_move(race, targets, event["move"])
    #随机位置事件：有值则让目标移动到指定位置
        if event["track_to_location_exist"] == 1:
            move_to = event["track_to_location"]
            event_track_to_location(race, targets, move_to)
    #随机位置事件：为1则让目标随机位置（位置范围为可设定值，见setting.py）
        if event["track_random_location"] == 1:
            event_track_random_location(race, targets)
    #buff持续时间调整事件：目标所有buff增加/减少回合数
        if event["buff_time_add"] != 0:
            event_buff_time_add(race, targets, event["buff_time_add"])
    #删除buff事件：下回合删除目标含特定buff_tag的所有buff
        if event["del_buff"] != f"":
            del_buffs = event["del_buff"]
            event_del_buffs(race, targets, del_buffs)
    #换位事件：值为1则与目标更换位置 （仅target为1,6时生效）
        if event["track_exchange_location"] == 1 and (target == 1 or target == 6):
            event_track_exchange_location(race, horse_i, targets[0])
    #一次性随机事件
        if event["random_event_once"] != []:
            random_event_once = event["random_event_once"]
            random_event_once_num = len(random_event_once)
            for i in targets:
                for j in range(0, random_event_once_num):
                    random_event_once_rate = random.randint(0, random_event_once[random_event_once_num - 1][0])
                    if random_event_once_rate <= random_event_once[j][0]:
                        event_once = random_event_once[j][1]
                        break
                describe += event_main(race, i, event_once, 1)
    #===============以下为永久事件===============
    #buff_tag，死亡：为1则目标死亡，此参数生成的buff默认持续999回合
    #buff_tag：die的自定义名称，不填为“死亡”
        if event["die"] == 1:
            event_die(race, targets, event["die_name"])
    #buff_tag，离开：为1则目标离开，此参数生成的buff默认持续999回合
    #buff_tag：away的自定义名称，不填为“离开”
        if event["away"] == 1:
            event_away(race, targets, event["away_name"])
    #==============================连锁事件预留位置，暂时没做

    #===============以下为buff事件===============
    #"rounds": 0,                #buff持续回合数
    #"name": "xxx",              #buff名称，turn值>0时为必要值
    #"move_max": 0,              #该buff提供马儿每回合位移值区间的最大值（move_max若小于move_min，则move_max以move_min值为准）
    #"move_min": 0,              #该buff提供马儿每回合位移值区间的最小值
    #"locate_lock": 0,           #buff_tag，止步：若为1则目标无法移动
    #"vertigo": 0,               #buff_tag，眩晕：若为1则目标无法移动，且不主动执行事件（暂定）
    #"hiding": 0,                #buff_tag，隐身：不显示目标移动距离及位置
    #"other_buff": ["buff1", "buff2", ....]
    #                            #自定义buff_tag，仅标识用buff_tag填写处，也可以填入常规buff_tag并正常生效
    #"random_event": [[概率值1, {事件}], [概率值2, {事件}], ......],
                                #此为持续性随机事件，以buff形式存在，部分详见文末
        if event["rounds"] > 0:
            rounds = event["rounds"]
            buffs = []
            buff_name = event["name"]
            move_max = event["move_max"]
            move_min = event["move_min"]
            if event["locate_lock"] == 1:
                buffs.append("locate_lock")
            if event["vertigo"] == 1:
                buffs.append("locate_lock")
                buffs.append("vertigo")
            if event["hiding"] == 1:
                buffs.append("hiding")
            if event["other_buff"] != []:
                buffs.extend(event["other_buff"])
            event_in_buff = event["random_event"]
            event_add_buff(race, targets, buff_name, rounds, buffs, move_min, move_max, event_in_buff)
    #===============以下为延迟事件===============
    #延迟事件（以当前事件的targets为发起人的事件）：前者为多少回合后，需>1
        delay_event = event["delay_event"]
        if delay_event != []:
            event_delay_rounds = delay_event[0]
            if event_delay_rounds > 1:
                event_delay = delay_event[1]
                for i in targets:
                    race.player[i].delay_events.append([race.round + event_delay_rounds, event_delay])
    #延迟事件（以当前事件发起人为发起人的事件）：前者为多少回合后，需>1
        delay_event_self = event["delay_event_self"]
        if delay_event_self != []:
            event_delay_rounds_self = delay_event_self[0]
            if event_delay_rounds_self > 1:
                event_delay_self = delay_event_self[1]
                race.player[horse_i].delay_events.append([race.round + event_delay_rounds_self, event_delay_self])

    #===============以下同步事件===============
    #同步事件（以当前事件的targets为发起人的事件），执行此事件后立马执行该事件
        another_event = event["another_event"]
        if another_event != {}:
            for i in targets:
                describe += event_main(race, i, another_event, 1)
    #同步事件（以当前事件发起人为发起人的事件），执行此事件后立马执行该事件
        another_event_self = event["another_event_self"]
        if another_event_self != {}:
            describe += event_main(race, horse_i, another_event_self, 1)
    #==========永久事件2，换赛道/加马==========
    #增加一匹马事件
        if event["add_horse"] != {}:
            add_horse_event = event["add_horse"]
            add_horse_name = add_horse_event["horsename"]
            add_horse_id = add_horse_event["owner"]
            try:
                add_horse_uid = add_horse_event["uid"]
            except KeyError:
                add_horse_uid = 114514
            try:
                add_horse_location = add_horse_event["location"]
            except KeyError:
                add_horse_location = 0
            logger.info(f'创建马{add_horse_name},{str(add_horse_uid)}, {add_horse_id}')
            race.add_player(add_horse_name, add_horse_uid, add_horse_id, add_horse_location, race.round)
    #替换一匹马事件
        replace_event = event["replace_horse"]
        if replace_event != {}:
            if target == 0 or target == 1 or target == 4 or target == 6:
                try:
                    replace_name = replace_event["horsename"]
                except KeyError:
                    replace_name = race.player[targets[0]].horse
                try:
                    replace_id = replace_event["owner"]
                except KeyError:
                    replace_id = race.player[targets[0]].player
                try:
                    replace_uid = replace_event["uid"]
                except KeyError:
                    replace_uid = race.player[targets[0]].playeruid
                logger.info(f'替换事件{replace_name}, {str(replace_uid)}, {replace_id}')
                race.player[targets[0]].replace_horse_ex(replace_name, replace_uid, replace_id)
        return describe
    except:
        logger.info(f"事件名 {event_name} 执行故障")
        return f"事件名 {event_name} 执行故障"
#===============子函数区===============
#复活对象
def event_live(race, targets):
    for i in targets:
        if race.player[i].find_buff("die") == True:
            race.player[i].del_buff("die")
        logger.info(f'{race.player[i].horse} 复活了')

#位移对象
def event_move(race, targets, move):
    for i in targets:
        race.player[i].location_move_event(move)
        logger.info(f'{race.player[i].horse} 移动了{str(move)}')

#移动对象至特定位置
def event_track_to_location(race, targets, move_to):
    for i in targets:
        race.player[i].location_move_to_event(move_to)
        logger.info(f'{race.player[i].horse} 移动到了指定位置{str(move_to)}')

#移动对象至随机位置
def event_track_random_location(race, targets):
    for i in targets:
        move_to = random.randint(setting_random_min_length, setting_random_max_length)
        race.player[i].location_move_to_event(move_to)
        logger.info(f'{race.player[i].horse} 移动到了随机位置{str(move_to)}')

#buff持续时间变更
def event_buff_time_add(race, targets, time_add):
    for i in targets:
        race.player[i].buff_addtime(time_add)
        logger.info(f'{race.player[i].horse} 的buff事件增加了{str(time_add)}')

#移除buff
def event_del_buffs(race, targets, del_buffs):
    for i in targets:
        for j in del_buffs:
            race.player[i].del_buff(j)
            logger.info(f'{race.player[i].horse} 移除了buff：{j}')
#马儿互换位置
def event_track_exchange_location(race, a, b):
    x = race.player[a].location
    race.player[a].location_move_to_event(race.player[b].location)
    race.player[b].location_move_to_event(x)
    logger.info(f'{race.player[a].horse} 和{race.player[b].horse}互换位置')


#死亡事件
def event_die(race, targets, buff_name):
    for i in targets:
        if race.player[i].find_buff("die") == False:
            race.player[i].add_buff(buff_name, race.round + 1, 9999, ["die"])
            logger.info(f'死亡事件判定 {race.player[i].horse} 死了')

#离开事件
def event_away(race, targets, buff_name):
    for i in targets:
        if race.player[i].find_buff("away") == False:
            race.player[i].add_buff(buff_name, race.round + 1, 9999, ["away"])
        logger.info(f'{race.player[i].horse} 离开')

#增加buff事件
def event_add_buff(race, targets, buff_name, rounds, buffs, move_min, move_max, event_in_buff):
    for i in targets:
        if race.player[i].find_buff(buff_name) == False:
            logger.info(f'{race.player[i].horse} 增加了 buff: {buff_name}, 第{str(race.round + 1)}~{str(race.round + rounds)}回合')
        else:
            race.player[i].del_buff(buff_name)
        race.player[i].add_buff(buff_name, race.round + 1, race.round + rounds, buffs, move_min, move_max, event_in_buff)




import math
import time
import json
import os
import io
from nonebot import on_command
from nonebot.adapters.onebot.v11 import Bot, GroupMessageEvent, MessageEvent, Message, MessageSegment
from nonebot.permission import SUPERUSER
from nonebot.params import CommandArg
from nonebot.log import logger
from .start import *
from .race_group import race_group
from .setting import  *
from nonebot_plugin_imageutils import Text2Image

__zx_plugin_name__ = "赛马"
__plugin_usage__ = """
usage：
    第一位玩家发起活动，指令：赛马创建
    加入赛马活动指令：赛马加入 [你的马儿名称]
    开始指令：赛马开始
    赛马超时重置指令：赛马重置

    管理员指令：赛马暂停/赛马清空
                    赛马事件重载
""".strip()
__plugin_des__ = "真寻小赌场-赛马场"
__plugin_cmd__ = ["赛马创建/加入[名称]/开始"]
__plugin_type__ = ("真寻小赌场",)
__plugin_version__ = 2.0
__plugin_author__ = "冥乐"
__plugin_settings__ = {
    "level": 5,
    "default_status": True,
    "limit_superuser": False,
    "cmd": ["赛马"],
}
events_list = []
#开场加载
driver=get_driver()
@driver.on_startup
async def events_read():
    global events_list
    events_list = await load_dlcs()


RaceNew = on_command("赛马创建", priority=5, block=True)
RaceJoin = on_command("赛马加入",priority=5, block=True)
RaceStart = on_command("赛马开始", priority=5, block=True)
RaceReStart = on_command("赛马重置", priority=5, block=True)
RaceStop = on_command("赛马暂停", priority=5, permission=SUPERUSER, block=True)
RaceClear = on_command("赛马清空", priority=5, permission=SUPERUSER, block=True)
RaceReload = on_command("赛马事件重载", priority=5, permission=SUPERUSER, block=True)
race = {}

@RaceNew.handle()
async def _(bot: Bot, event: GroupMessageEvent, arg: Message = CommandArg()):
    global race
    group = event.group_id
    try:
        if race[group].start == 0 and time.time() - race[group].time < 300:
            out_msg = f'> 创建赛马比赛失败!\n> 原因:赫尔正在打扫赛马场...\n> 解决方案:等赫尔打扫完...\n> 可以在{str(setting_over_time - time.time() + race[group].time)}秒后输入 赛马重置'
            await RaceNew.finish(out_msg)
        elif race[group].start == 1 :
            await RaceNew.finish(f"一场赛马正在进行中")
            await RaceNew.finish()
    except KeyError:
        pass
    race[group] = race_group()
    await RaceNew.finish(f'> 创建赛马比赛成功\n> 输入 赛马加入+名字 即可加入赛马')

@RaceJoin.handle()
async def _(bot: Bot, event: GroupMessageEvent, arg: Message = CommandArg()):
    global race, max_player
    msg = arg.extract_plain_text().strip()
    uid = event.user_id
    group = event.group_id
    player_name = event.sender.card if event.sender.card else event.sender.nickname
    try:
        race[group]
    except KeyError:
        await RaceJoin.finish( f"赛马活动未开始，请输入“赛马创建”开场")
    try:
        if race[group].start == 1 or race[group].start == 2:
            await RaceJoin.finish()
    except KeyError:
        await RaceJoin.finish()
    if race[group].query_of_player() >= max_player:
        await RaceJoin.finish( f"> 加入失败\n> 原因:赛马场就那么大，满了满了！" )
    if race[group].is_player_in(uid) == True:
        await RaceJoin.finish( f"> 加入失败\n> 原因:您已经加入了赛马场!")
    if msg:
        horse_name = msg
    else:
        await RaceJoin.finish(f"请输入你的马儿名字", at_sender=True)
    race[group].add_player(horse_name, uid, player_name)
    out_msg = f'> 加入赛马成功\n> 赌上马儿性命的一战即将开始!\n> 赛马场位置:{str(race[group].query_of_player())}/{str(max_player)}'
    await RaceJoin.finish(out_msg, at_sender=True)

@RaceStart.handle()
async def _(bot: Bot, event: GroupMessageEvent, arg: Message = CommandArg()):
    global race
    global events_list
    group = event.group_id
    try:
        if race[group].query_of_player() == 0:
            await RaceStart.finish()
    except KeyError:
        await RaceStart.finish()
    try:
        if race[group].start == 0 or race[group].start == 2:
            if len(race[group].player) >= min_player:
                race[group].start_change(1)
            else:
                await RaceStart.finish(f'> 开始失败\n> 原因:赛马开局需要最少{str(min_player)}人参与', at_sender=True)
        elif race[group].start == 1:
            await RaceStart.finish()
    except KeyError:
        await RaceStart.finish()
    race[group].time = time.time()
    while race[group].start == 1:
#显示器初始化
        display = f""
# 回合数+1
        race[group].round_add()
#移除超时buff
        race[group].del_buff_overtime()
#马儿全名计算
        race[group].fullname()
#回合事件计算
        display += race[group].event_start(events_list)
#马儿移动
        race[group].move()
#场地显示
        display += race[group].display()
        logger.info(f'事件输出:\n {display}')
        ima = Text2Image.from_text(display, 20, fill="black").to_image("white")
        imgByteArr = io.BytesIO()
        ima.save(imgByteArr,format="PNG")
        await RaceStart.send(MessageSegment.image(imgByteArr))
        time.sleep(2)
#全员失败计算
        if race[group].is_die_all():
            del race[group]
            await RaceStart.finish("比赛已结束，鉴定为无马生还")
#全员胜利计算
        winer = race[group].is_win_all()
        if winer != f"":
            await RaceStart.send(f'> 比赛结束\n> 赫尔正在为您生成战报...')
            time.sleep(2)
            del race[group]
            msg = "比赛已结束，胜者为：" + winer
            await RaceStart.finish(msg)
        time.sleep(4)

@RaceReStart.handle()
async def _(bot: Bot, event: MessageEvent, arg: Message = CommandArg()):
    global race
    group = event.group_id
    time_key = math.ceil(time.time() - race[group].time)
    if time_key >= setting_over_time:
        del race[group]
        await RaceReStart.finish(f'超时{str(setting_over_time)}秒，准许重置赛马场')
    await RaceReStart.finish(f'未超时{str(setting_over_time)}秒，目前为{str(time_key)}秒，未重置')

@RaceStop.handle()
async def _(bot: Bot, event: GroupMessageEvent, arg: Message = CommandArg()):
    global race
    group = event.group_id
    race[group].start_change(2)

@RaceClear.handle()
async def _(bot: Bot, event: MessageEvent, arg: Message = CommandArg()):
    global race
    group = event.group_id
    del race[group]

@RaceReload.handle()
async def _(bot: Bot, event: MessageEvent, arg: Message = CommandArg()):
    global events_list
    logs = f""
    files = os.listdir(os.path.dirname(__file__) + '/events')
    for file in files:
        try:
            with open(f'{os.path.dirname(__file__)}/events/{file}', "r", encoding="utf-8") as f:
                logger.info(f'加载事件文件：{file}')
                events = deal_events(json.load(f))
                events_list.extend(events)
            logger.info(f"加载 {file} 成功")
            logs += f'加载 {file} 成功\n'
        except:
            logger.info(f"加载 {file} 失败！失败！失败！")
            logs += f"加载 {file} 失败！失败！失败！\n"
    await RaceReload.finish(logs)




from nonebot.log import logger
from nonebot import get_driver
import json
import os

async def load_dlcs():
    events_list = []
    logs = f""
    files = os.listdir(os.path.dirname(__file__) + '/events')
    for file in files:
        try:
            with open(f'{os.path.dirname(__file__)}/events/{file}', "r", encoding="utf-8") as f:
                logger.info(f'加载事件文件：{file}')
                events = deal_events(json.load(f))
                events_list.extend(events)
            logger.info(f"加载 {file} 成功")
            logs += f'加载 {file} 成功\n'
        except:
            logger.info(f"加载 {file} 失败！失败！失败！")
            logs += f"加载 {file} 失败！失败！失败！\n"
    return events_list



def deal_events(events):
    events_out = []
    for i in range(0,len(events)):
        event_i = deal(events[i])
        if event_i != {}:
            events_out.append(event_i)
    return events_out

def deal(event):
    event_out = {}
    # 读取事件限定值
    try:
        event_out["race_only"] = event["race_only"]
        event_out["race_only_exist"] = 1
    except KeyError:
        event_out["race_only_exist"] = 0
    # 读取 event_name 无则未知事件
    try:
        event_out["event_name"] = event["event_name"]
    except KeyError:
        event_out["event_name"] = "未知事件"
    # 读取 describe 事件描述，无描述则空
    try:
        event_out["describe"] = event["describe"]
    except KeyError:
        event_out["describe"] = f""
    # 读取 target 目标，无则销毁事件
    try:
        event_out["target"] = event["target"]
    except KeyError:
        return {}
    # 判定target_is_buff
    try:
        event_out["target_is_buff"] = event["target_is_buff"]
    except KeyError:
        event_out["target_is_buff"] = f""
    # 判定target_no_buff
    try:
        event_out["target_no_buff"] = event["target_no_buff"]
    except KeyError:
        event_out["target_no_buff"] = f""
    # ===============以下为一次性事件===============
    # 复活事件：为1则目标复活
    try:
        event_out["live"] = event["live"]
    except KeyError:
        event_out["live"] = 0
    # 位移事件：目标立即进行相当于参数值的位移
    try:
        event_out["move"] = event["move"]
    except KeyError:
        event_out["move"] = 0
    # 随机位置事件：有值则让目标移动到指定位置
    try:
        event_out["track_to_location"] = event["track_to_location"]
        event_out["track_to_location_exist"] = 1
    except KeyError:
        event_out["track_to_location_exist"] = 0
    # 随机位置事件：为1则让目标随机位置（位置范围为可设定值，见setting.py）
    try:
        event_out["track_random_location"] = event["track_random_location"]
    except KeyError:
        event_out["track_random_location"] = 0
    # buff持续时间调整事件：目标所有buff增加/减少回合数
    try:
        event_out["buff_time_add"] = event["buff_time_add"]
    except KeyError:
        event_out["buff_time_add"] = 0
    # 删除buff事件：下回合删除目标含特定buff_tag的所有buff
    try:
        event_out["del_buff"] = event["del_buff"]
    except KeyError:
        event_out["del_buff"] = f""
    # 换位事件：值为1则与目标更换位置 （仅target为1,6时生效）
    try:
        event_out["track_exchange_location"] = event["track_exchange_location"]
    except KeyError:
        event_out["track_exchange_location"] = 0
    # 一次性随机事件
    try:
        event_out["random_event_once"] = event["random_event_once"]
        if event_out["random_event_once"] != []:
            for i in range(0, len(event_out["random_event_once"])):
                event_out["random_event_once"][i][1] = deal(event_out["random_event_once"][i][1])
    except KeyError:
        event_out["random_event_once"] = []
    # ===============以下为永久事件===============
    # buff_tag，死亡：为1则目标死亡，此参数生成的buff默认持续999回合
    # buff_tag：die的自定义名称，不填为“死亡”
    try:
        event_out["die"] = event["die"]
    except KeyError:
        event_out["die"] = 0
    try:
        event_out["die_name"] = event["die_name"]
    except KeyError:
        event_out["die_name"] = "死亡"
    # buff_tag，离开：为1则目标离开，此参数生成的buff默认持续999回合
    # buff_tag：away的自定义名称，不填为“离开”
    try:
        event_out["away"] = event["away"]
    except KeyError:
        event_out["away"] = 0
    try:
        event_out["away_name"] = event["away_name"]
    except KeyError:
        event_out["away_name"] = "离开"
    # ==============================连锁事件预留位置，暂时没做

    # ===============以下为buff事件===============
    # "rounds": 0,                #buff持续回合数
    # "name": "xxx",              #buff名称，turn值>0时为必要值
    # "move_max": 0,              #该buff提供马儿每回合位移值区间的最大值（move_max若小于move_min，则move_max以move_min值为准）
    # "move_min": 0,              #该buff提供马儿每回合位移值区间的最小值
    # "locate_lock": 0,           #buff_tag，止步：若为1则目标无法移动
    # "vertigo": 0,               #buff_tag，眩晕：若为1则目标无法移动，且不主动执行事件（暂定）
    # "hiding": 0,                #buff_tag，隐身：不显示目标移动距离及位置
    # "other_buff": ["buff1", "buff2", ....]
    #                            #自定义buff_tag，仅标识用buff_tag填写处，也可以填入常规buff_tag并正常生效
    # "random_event": [[概率值1, {事件}], [概率值2, {事件}], ......],
    # 此为持续性随机事件，以buff形式存在，部分详见文末
    try:
        event_out["rounds"] = event["rounds"]
    except KeyError:
        event_out["rounds"] = 0
    try:
        event_out["name"] = event["name"]
    except KeyError:
        event_out["name"] = "未命名buff"
    try:
        event_out["move_max"] = event["move_max"]
    except KeyError:
        event_out["move_max"] = 0
    try:
        event_out["move_min"] = event["move_min"]
    except KeyError:
        event_out["move_min"] = 0
    if event_out["move_max"] < event_out["move_min"]:
        event_out["move_max"] = event_out["move_min"]
    try:
        event_out["locate_lock"] = event["locate_lock"]
    except KeyError:
        event_out["locate_lock"] = 0
    try:
        event_out["vertigo"] = event["vertigo"]
    except KeyError:
        event_out["vertigo"] = 0
    try:
        event_out["hiding"] = event["hiding"]
    except KeyError:
        event_out["hiding"] = 0
    try:
        event_out["other_buff"] = event["other_buff"]
    except KeyError:
        event_out["other_buff"] = []
    try:
        event_out["random_event"] = event["random_event"]
        if event_out["random_event"] != []:
            for i in range(0, len(event_out["random_event"])):
                event_out["random_event"][i][1] = deal(event_out["random_event"][i][1])
    except KeyError:
        event_out["random_event"] = []
    # ===============以下为延迟事件===============
    # 延迟事件（以当前事件的targets为发起人的事件）：前者为多少回合后，需>1
    try:
        delay_event = event["delay_event"]
        if event_out["delay_event"] != []:
            event_out["delay_event"][1] = deal(event_out["delay_event"][1])
    except KeyError:
        event_out["delay_event"] = []
    # 延迟事件（以当前事件发起人为发起人的事件）：前者为多少回合后，需>1
    try:
        event_out["delay_event_self"] = event["delay_event_self"]
        if event_out["delay_event_self"] != []:
            event_out["delay_event_self"][1] = deal(event_out["delay_event_self"][1])
    except KeyError:
        event_out["delay_event_self"] = []
    # ===============以下同步事件===============
    # 同步事件（以当前事件的targets为发起人的事件），执行此事件后立马执行该事件
    try:
        event_out["another_event"] = event["another_event"]
        event_out["another_event"] = deal(event_out["another_event"])
    except KeyError:
        event_out["another_event"] = {}
    # 同步事件（以当前事件发起人为发起人的事件），执行此事件后立马执行该事件
    try:
        event_out["another_event_self"] = event["another_event_self"]
        event_out["another_event_self"] = deal(event_out["another_event_self"])
    except KeyError:
        event_out["another_event_self"] = {}
    # ==========永久事件2，换赛道/加马==========
    # 增加一匹马事件
    try:
        event_out["add_horse"] = event["add_horse"]
    except KeyError:
        event_out["add_horse"] = {}
    # 替换一匹马事件
    try:
        event_out["replace_horse"] = event["replace_horse"]
    except KeyError:
        event_out["replace_horse"] = {}
    return event_out



import random
import math
import time
from .horse import horse
from .events_main import event_main
from .setting import  *

class race_group:
#初始化
    def __init__(self):
        self.player = []
        self.round = 0
        self.start = 0
        self.time = time.time()
        self.race_only_keys = []
#start指示器变更 0为马儿进场未开始，1为开始，2为暂停（测试用）
    def start_change(self, key):
        self.start = key
#增加赛马位
    def add_player(self, horsename = "the_horse", uid = 114514, id = "the_player",location = 0, round = 0):
        self.player.append(horse(horsename, uid, id, location, round))
#赛马位数量查看
    def query_of_player(self):
        return len(self.player)
#查找有无玩家
    def is_player_in(self, uid):
        for i in range(0, len(self.player)):
            if self.player[i].playeruid == uid:
                return True
        return False
#回合开始，回合数+1
    def round_add(self):
        self.round += 1
        for i in range(0, len(self.player)):
            self.player[i].round = self.round
            self.player[i].location_add_move = 0
#所有马儿移除多余buff
    def del_buff_overtime(self):
        for i in range(0, len(self.player)):
            self.player[i].del_buff_overtime(self.round)
#所有马儿buff+名称计算
    def fullname(self):
        for i in range(0, len(self.player)):
            self.player[i].fullname()
#所有马儿移动，移动计算已包含死亡/离开/止步判定
    def move(self):
        for i in range(0, len(self.player)):
            self.player[i].location_move()
#所有马儿数据显示（须先移动)
    def display(self):
        display = ""
        for i in range(0, len(self.player)):
            display += self.player[i].display()
        return display
#所有马儿是否死亡/离开
    def is_die_all(self) -> bool:
        for i in range(0, len(self.player)):
            if ( self.player[i].is_die() == False ) and ( self.player[i].is_away() == False ):
                return False
        return True
#所有马儿是否到终点
    def is_win_all(self):
        win_name = f""
        for i in range(0, len(self.player)):
            if self.player[i].location >= setting_track_length:
                win_name += "\n" + self.player[i].player
        return win_name
#事件触发
    def event_start(self, events_list):
        event_display = f""
        #延时事件触发：
        for i in range(0, len(self.player)):
            if len(self.player[i].delay_events) > 0:
                for j in range(len(self.player[i].delay_events)-1, -1, -1):
                    if self.player[i].delay_events[j][0] == self.round:
                        display_0 = event_main(self, i, self.player[i].delay_events[j][1], 1) + "\n"
                        if display_0 != "\n":
                            event_display += display_0
                        del self.player[i].delay_events[j]
        #buff随机事件触发
        for i in range(0, len(self.player)):
            for j in range(0, len(self.player[i].buff)):
                if self.player[i].buff[j][5] != []:
                    event_in_buff = self.player[i].buff[j][5]
                    event_in_buff_num = len(event_in_buff)
                    event_in_buff_rate = random.randint(0, event_in_buff[event_in_buff_num - 1][0])
                    for k in range(0, event_in_buff_num):
                        if event_in_buff_rate <= event_in_buff[k][0]:
                            event_in_buff_x = event_in_buff[k][1]
                            break
                    display_0 = event_main(self, i, event_in_buff_x, 1) + "\n"
                    if display_0 != "\n":
                        event_display += display_0
                        #随机事件判定：
        events_num = len(events_list)
        for i in range(0, len(self.player)):
            event_id = random.randint(0, math.ceil(1000 * events_num / event_rate) - 1)
            if event_id < events_num:
                event = events_list[event_id]
                display_0 = event_main(self, i, event) + "\n"
                if display_0 != "\n":
                    event_display += display_0
        return event_display
#事件唯一码查询
    def is_race_only_key_in(self, key):
        try:
            self.race_only_keys.index(key)
            return True
        except ValueError:
            return False
#事件唯一码增加
    def add_race_only_key(self, key):
        self.race_only_keys.append(key)
